<?php
namespace Wovnio\Wovnwp;

require_once( 'constants.php' );
require_once( 'wp-classes/swapper.php' );
require_once( 'wp-classes/request_handlers/request_handler_factory.php' );
require_once( 'wp-classes/headers_transformer.php' );
require_once( 'wp-classes/html_converter.php' );
require_once( 'wp-classes/url.php' );
require_once( 'wp-classes/lang.php' );
require_once( 'wp-classes/wovn_options.php' );

use Wovnio\Wovnwp\WpClasses\RequestHandlers\RequestHandlerFactory;
use Wovnio\Wovnwp\WpClasses\HeadersTransformer;

class Translate
{
	private $ob_level;
	private $swapper;

	public function __construct(&$localization_decider)
	{
		$this->ob_level = PHP_INT_MAX;

		// Multiple site wordpress needs the correct `$_SERVER["REQUEST_URI"]` to get the
		// blog_id (site id), and this happen before the request is intercepted by the plugin
		// So we strip language part from  `$_SERVER["REQUEST_URI"]` in wordpress's `index.php`,
		// where request handling start and keep the original in `$_SERVER["WOVN_REQUEST_URI"]`
		// to use later in the translation process
		if (isset($_SERVER["WOVN_REQUEST_URI"])) {
			$request_uri = $_SERVER["WOVN_REQUEST_URI"];
		} else {
			$request_uri = isset($_SERVER["REQUEST_URI"]) ? $_SERVER["REQUEST_URI"] : '';
		}

		$accessed_url = (isset($_SERVER['HTTPS']) ? 'https' : 'http') . '://' . $_SERVER['HTTP_HOST'] . $request_uri;
		$wovn_options = WpClasses\WovnOptions::get_instance()->get_all();
		$lang_param_name = $wovn_options['wovn_lang_param_name'];
		$request_variables = array(
			"HTTP_HOST" => isset($_SERVER["HTTP_HOST"]) ? $_SERVER["HTTP_HOST"] : '',
			"REQUEST_URI" => $request_uri,
			"QUERY_STRING" => isset($_SERVER["QUERY_STRING"]) ? $_SERVER["QUERY_STRING"] : '',
			"WOVN_QUERY" => isset($_GET[$lang_param_name]) ? $_GET[$lang_param_name] : ''
		);
		$this->swapper = new WpClasses\Swapper(
			$accessed_url,
			$wovn_options,
			$request_variables,
			RequestHandlerFactory::get_best_available_request_handler($wovn_options),
      $localization_decider,
			new HeadersTransformer()
		);

		add_action( 'plugins_loaded', array( $this, 'set_lang_and_original_url' ), 1 );
		add_action( 'plugins_loaded', array( $this, 'validate' ), 2 );
		// Do not translate in admin page
		if($localization_decider->can_intercept()) {
			add_action( 'plugins_loaded', array( $this, 'buffer_start' ), 3 );
			add_action( 'shutdown', array( $this, 'buffer_stop' ), 1000 );
		}
	}

	function set_lang_and_original_url() {
		$changed_uri = $this->swapper->set_lang_and_original_url();
		if ($changed_uri) {
			$_SERVER['REQUEST_URI'] = $changed_uri;
		}
	}

	// validation
	function validate() {
		$this->swapper->validate();
	}

	function buffer_start() {
		ob_start(array( $this, 'buffer_callback' ));
		$this->ob_level = ob_get_level();
	}

	function buffer_callback($buffer){
		return $this->swapper->swap_html($buffer);
	}

	function buffer_stop() {
		while(ob_get_level() > $this->ob_level)  {
			ob_end_flush();
		}
	}
}
