<?php
namespace Wovnio\Wovnwp\WpClasses;

require_once( 'url.php' );

class RedirectionHandler {
	private $url_pattern = null;
	private $source_lang = null;
	private $target_lang = null;
	private $lang_param_name = null;
	private $exit_on_redirection = false;

	public function __construct($options, $localization_decider, $exit_on_redirection = true) {
		if (isset($_SERVER['HTTP_REFERER'])) {
			$this->url_pattern = $options['wovn_url_pattern_name'];
			$this->source_lang = $options['wovn_default_lang'];
			$this->lang_param_name = $options['wovn_lang_param_name'];
			$this->target_lang = Url::path_lang_code($_SERVER['HTTP_REFERER'], $this->url_pattern, $options['wovn_supported_langs'], $this->source_lang, $this->lang_param_name);
			$this->exit_on_redirection = $exit_on_redirection;
			$this->localization_decider = $localization_decider;
		}
	}

	public function is_active() {
		return $this->url_pattern && $this->source_lang && $this->target_lang;
	}

	public function transform_redirect_url($url) {
		if (!$this->is_active() || !$this->localization_decider->can_change_url($url)) return $url;

		return htmlentities(Url::add_lang_code($url, $this->url_pattern, $this->target_lang, $this->source_lang, $this->lang_param_name));
	}

	public function audit_headers_and_transform_redirection() {
		if (!$this->is_active()) return;

		$response_headers = headers_list();

		foreach (array('location', 'Location') as $location_header) {
			if (array_key_exists($location_header, $response_headers)) {
				$redirect_url = $this->transform_redirect_url($response_headers[$location_header]);

				header("Location: {$redirect_url}");
				if ($this->exit_on_redirection) exit;
			}
		}
	}
}
